import { world, system, ItemStack } from "@minecraft/server"
import { hasEquipmentArtifact, randomInt } from '../utils/functions.js'


system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        const equippableComponent = player.getComponent('equippable');
        const equipment = {
            mainhand: equippableComponent.getEquipment('Mainhand'),
            head: equippableComponent.getEquipment('Head'),
            chest: equippableComponent.getEquipment('Chest'),
            legs: equippableComponent.getEquipment('Legs'),
            feet: equippableComponent.getEquipment('Feet')
        };

        if (hasEquipmentArtifact(player, equipment.legs, 'artifacts:crystal_heart')) {
            player.addEffect('health_boost', 20 * 2, { amplifier: 2, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.head, 'artifacts:night_vision_goggles')) {
            player.addEffect('night_vision', 20 * 15, { amplifier: 0, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.head, 'artifacts:snorkel')) {
            player.addEffect('water_breathing', 20 * 2, { amplifier: 0, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.head, 'artifacts:villager_hat')) {
            player.addEffect('village_hero', 20 * 2, { amplifier: 0, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.head, 'artifacts:cowboy_hat')) {
            const entity = player.getComponent("riding")?.entityRidingOn;
            entity?.addEffect('speed', 20 * 5, { amplifier: 1, showParticles: false })

        }
        if (hasEquipmentArtifact(player, equipment.head, 'artifacts:plastic_drinking_hat') || hasEquipmentArtifact(player, equipment.head, 'artifacts:novelty_drinking_hat')) {
            //world.sendMessage('hola yo')
            player.addEffect('saturation', 20 * 2, { amplifier: 0, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.mainhand, 'artifacts:umbrella') && !player.isOnGround) {
            player.addEffect('slow_falling', 20 * 2, { amplifier: 0, showParticles: false })
            //player.playAnimation('animation.has_umbrella', { blendOutTime: 0.2, stopExpression: 'q.is_on_ground || q.is_in_water' })
            //player.applyKnockback(0, 0, 0, 0.001)
        }
        if (hasEquipmentArtifact(player, equipment.legs, 'artifacts:helium_flamingo')) {
            if (player.isJumping && player.isSprinting && !player.hasTag('artifacts:flamingo_effect')) {
                //world.sendMessage('init flamingo_effect')
                player.addTag('artifacts:flamingo_effect')
                player.addEffect('levitation', 20 * 10, { amplifier: 2, showParticles: false })
                player.playAnimation('animation.custom_swim', { stopExpression: 'q.is_on_ground || q.is_in_water' })
                system.runTimeout(() => {
                    player.playAnimation('animation.humanoid.base_pose')
                }, 20 * 10)
                system.runTimeout(() => {
                    player.removeTag('artifacts:flamingo_effect')
                }, 20 * 25)//15+10
            }
        }
        if (hasEquipmentArtifact(player, equipment.feet, 'artifacts:rooted_boots')) {

            let { x, y, z } = player.location
            let block = player.dimension.getBlock({ x: x, y: y - 1, z: z })

            if (block.typeId.includes('grass')) {

                let time = player.getDynamicProperty('artifact:time') || 0;
                player.setDynamicProperty('artifact:time', time + 1);

                //world.sendMessage(`${time.toString()}`)

                if (time >= 20) {
                    const apple = new ItemStack('minecraft:apple')
                    //world.sendMessage('yum')
                    player.eatItem(apple)
                    player.setDynamicProperty('artifact:time', undefined)

                }

            }
        }

        const hasCharmOfShrinking = hasEquipmentArtifact(player, equipment.chest, 'artifacts:charm_of_shrinking');
        const isShrinking = player.hasTag('is_shrinking');

        if (hasCharmOfShrinking && !isShrinking) {
            player.addTag('is_shrinking');
            player.playAnimation('animation.small_player', { blendOutTime: 99999 });
        } else if (!hasCharmOfShrinking && isShrinking) {
            player.removeTag('is_shrinking');
            player.playAnimation('animation.humanoid.base_pose');
        }

        if (hasEquipmentArtifact(player, equipment.chest, 'artifacts:charm_of_sinking')) {
            player.addEffect('conduit_power', 20 * 15, { amplifier: 0, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.chest, 'artifacts:scarf_of_invisibility')) {
            player.addEffect('invisibility', 20 * 2, { amplifier: 0, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.chest, 'artifacts:digging_claws')) {
            player.addEffect('haste', 20 * 2, { amplifier: 0, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.feet, 'artifacts:bunny_hoppers')) {
            player.addEffect('jump_boost', 20 * 2, { amplifier: 2, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.feet, 'artifacts:running_shoes')) {
            player.addEffect('speed', 20 * 2, { amplifier: 1, showParticles: false })
        }
        if (hasEquipmentArtifact(player, equipment.feet, 'artifacts:kitty_slippers')) {
            let dimension = player.dimension
            let location = player.location
            let creeperEntity = dimension.getEntities({ type: "minecraft:creeper", closest: 1, maxDistance: 6, location: location })[0]
            if (creeperEntity != undefined) {
                dimension.spawnEntity('artifacts:dummy_cat', location)
            }
        }
        if (hasEquipmentArtifact(player, equipment.feet, 'artifacts:flippers') && player.isSwimming) {
            const view = player.getViewDirection();
            const knockbackStrength = 1.25;//1.5
            const yDirection = view.y > 0 ? 0.1 : -0.1;
            player.applyKnockback({ x: view.x * knockbackStrength, z: view.z * knockbackStrength }, yDirection);

        }
    }
}, 5);








//snowshoes
/* function vectorDistance(v1, v2) {
    const dx = v2.x - v1.x;
    const dy = v2.y - v1.y;
    const dz = v2.z - v1.z;
    return Math.sqrt(dx * dx + dy * dy + dz * dz);
}

system.runInterval(() => {
    const players = world.getAllPlayers()
    players.forEach(player => {
        const feetEquipment = player.getComponent('equippable').getEquipment('Feet')
        if (hasEquipmentArtifact(player, feetEquipment, 'artifacts:snowshoes')) {
            const { x, y, z } = player.location
            const dimension = player.dimension

            const block = dimension.getBlock({ x: x, y: y - 1, z: z })
            if (block.typeId != 'minecraft:powder_snow') {
                return
            }
            //if (block.typeId == 'minecraft:water')
            let previousEntity = dimension.getEntitiesAtBlockLocation(block.center())[0]

            if (previousEntity || player.isSneaking || player.isInWater) { return }

            const entity = dimension.spawnEntity('artifacts:dummy_platform', { x: block.center().x, y: block.center().y - 0.1, z: block.center().z })
            const tameableComponent = entity.getComponent('tameable')
            tameableComponent.tame(player)
        }
    })
}, 1)
world.afterEvents.dataDrivenEntityTrigger.subscribe(ev => {
    const { entity, eventId } = ev
    if (entity.typeId == 'artifacts:dummy_platform' && eventId == 'artifacts:try_despawn') {
        const tameableComponent = entity.getComponent('tameable')
        let owner = tameableComponent.tamedToPlayer
        const distance = vectorDistance(entity.location, owner.location)
        world.sendMessage(`La distancia hasta el owner es de: ${distance.toString()}`)
        if (!owner || distance > 3 || owner.isSneaking) {
            entity.remove()
        }
    }
})

 */