import { world, ItemStack, system } from "@minecraft/server"
import { hasEquipmentArtifact } from '../utils/functions.js'

function vectorDistance(v1, v2) {
    const dx = v2.x - v1.x;
    const dy = v2.y - v1.y;
    const dz = v2.z - v1.z;
    return Math.sqrt(dx * dx + dy * dy + dz * dz);
}

system.runInterval(() => {
    const players = world.getAllPlayers()
    players.forEach(player => {
        let feetEquipment = player.getComponent("equippable")?.getEquipment('Feet')
        if (hasEquipmentArtifact(player, feetEquipment, 'artifacts:aqua_dashers')) {
            const { x, y, z } = player.location
            const dimension = player.dimension

            for (let offsetX = -1; offsetX <= 1; offsetX++) {
                for (let offsetZ = -1; offsetZ <= 1; offsetZ++) {
                    const block = dimension.getBlock({ x: x + offsetX, y: y - 1, z: z + offsetZ })
                    let previousEntity = dimension.getEntitiesAtBlockLocation(block.center())[0]
                    if (block.typeId != 'minecraft:water' && !block.isWaterlogged) {
                        continue
                    }

                    //let previousEntity = dimension.getEntitiesAtBlockLocation(block.center())[0]
                    if (previousEntity || player.isSneaking || player.isInWater) { continue }

                    const entity = dimension.spawnEntity('artifacts:dummy_platform', { x: block.center().x, y: block.center().y - 0.1, z: block.center().z })
                    const tameableComponent = entity.getComponent('tameable')
                    tameableComponent.tame(player)
                }
            }
        }

    })
}, 1)

world.afterEvents.dataDrivenEntityTrigger.subscribe(ev => {
    const { entity, eventId } = ev
    if (entity.typeId == 'artifacts:dummy_platform' && eventId == 'artifacts:try_despawn') {
        const tameableComponent = entity.getComponent('tameable')
        let owner = tameableComponent.tamedToPlayer
        const distance = vectorDistance(entity.location, owner.location)
        //world.sendMessage(`La distancia hasta el owner es de: ${distance.toString()}`)
        if (!owner || distance > 2.5 || owner.isSneaking) {
            entity.remove()
        }
    }
})
