import { Entity, ItemStack, Player } from "@minecraft/server"

export function hasEquipmentInSpecificSlots(player, itemID, slots) {
    for (let slot of slots) {
        const inv = player.getComponent('inventory').container.getItem(slot)
        if (inv?.typeId == itemID) {
            return true;
        }
    }

    return false;
}
export function getRandomElement(array) {
    const indiceAleatorio = Math.floor(Math.random() * array.length);
    return array[indiceAleatorio];
}

/**
 * 
 * @param {Entity} entity 
 * @param {ItemStack} equiptment 
 * @param {string} itemID 
 * @returns boolean
 */
export function hasEquipmentArtifact(entity, equiptment, itemID) {
    if (equiptment?.typeId == itemID || entity.hasTag(`novelty:${itemID}`)) {
        return true;
    }
    return false;
}
export function hasEquipmentSlot(player, itemID, slot) {
    let getEquipment = player.getComponent('equippable').getEquipment(slot)
    if (getEquipment?.typeId == itemID) {
        return true;
    }
}

export function hasItem(player, itemId) {
    let inventory = player.getComponent("inventory");
    if (!inventory) {
        return false;
    }
    let container = inventory.container;
    for (let i = 0; i < container.size; i++) {
        let item = container.getItem(i);
        if (item?.typeId == itemId) {
            return true;
        }
    }
    return false;
}

export function clearMainhandItem(player) {
    if (player.matches({ gameMode: `creative` })) return
    player.getComponent("equippable").setEquipment("Mainhand", undefined)
}

export function updateItemAmount(item, player) {
    if (player.matches({ gameMode: `creative` })) return

    let equippable = player.getComponent("equippable")
    if (item.amount == 1) {
        equippable.setEquipment("Mainhand", undefined)
        return
    }
    item.amount -= 1
    equippable.setEquipment('Mainhand', item)
}

export function updateItemDurability(source, item, durabilityModifier = 1) {
    if (source.matches({ gameMode: `creative` })) return

    const equippable = source.getComponent("equippable");
    const durability = item.getComponent("durability");

    durability.damage += durabilityModifier;

    const maxDurability = durability.maxDurability
    const currentDamage = durability.damage
    if (currentDamage >= maxDurability) {

        source.playSound('random.break', { pitch: 1, location: source.location, volume: 1 })
        equippable.setEquipment("Mainhand", undefined);
    }
    else if (currentDamage < maxDurability) {

        equippable.setEquipment("Mainhand", item);
    }
}

export function randomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}
/* 
export function teleportPlayer(source) {
    let block = source.dimension.getBlockBelow(source.location, {
        includeLiquidBlocks: true,
    });
    let { x, y, z } = block.location;
    source.teleport({
        x: Math.floor(x) + 0.5,
        y: y + 0.5,
        z: Math.floor(z) + 0.5,
    });
}
 */